/* ********************************************************
 * 
 * 
 *      Example for the use of      uaru
 * 
 * (c) 2017 Mellspa
 * 
 * ******************************************************* */

#include <stdafx.h>
#include <fstream>
#include <sstream>
#include <uaru.h>

#define maxbuffsize 1024

using namespace std;

int main()
{

  // Downloading a page by http, user name and password are not used (up to now) for http	
  UARU u = ConnectURL("http://www.mellspa.de/",1,"optional_user","optional_password");
  //UARU u = ConnectURL("http://127.0.0.1:8000/mjpg/video.mjpg",1,"optional_user","optional_password");
  
  // you have got a handle for the URL. It is a handle to an UARU object.
  cout << "u: " << u << endl;
  
  // Open a file on the local file system (an other option for an example)  
  //UARU dest = ConnectURL("file:///uaruexample.cpp");
  
  // Writing the downloaded data into a local file. The second argument 0 indicates the write request.
  UARU dest = ConnectURL("file:///uaru_example.txt",0);
  
  
  
  // create a buffer to receive the content
  char buffer[maxbuffsize];
   
  
  // some variables to organize the download
  size_t currentbuffersize = 0;
  size_t transportbuffersize = 0; 
  UARUState uaru_state;
  
  
  
  // useful for MJPG streams
  size_t loadsize = 0;
  size_t partnbr = 0;
  ofstream* jpgfile = nullptr;
  
  // Read out repeatedly the source until UARU_EOF or UARU_ERROR. Use the handle you've got.
  for(;partnbr<10 && UARU_EOF>(uaru_state=GetData(u, buffer+currentbuffersize, transportbuffersize = maxbuffsize- currentbuffersize -1));)
  {
  
    
    
    // the transportbuffersize will be set by the function
    currentbuffersize += transportbuffersize;
    
    // in order to print it later set the last byte to 0; otherwise not needed
    buffer[currentbuffersize]=0;
    

    
    // If you 've got a multipart stream you will need to take care for some buffering
    if(uaru_state==UARU_MULTIPART)
    { // it is a multipart MJPG stream
      if(loadsize)
      {
	// Writes the buffer to the URL indicated by dest and the jpg-file. Instructive, not optimized.
	if(loadsize>=currentbuffersize)
	{
	  WriteData(dest,buffer,currentbuffersize);
	  if(jpgfile) jpgfile->write(buffer,currentbuffersize);
	  loadsize -= currentbuffersize;
	  currentbuffersize = 0;
	}
	else
	{
	  WriteData(dest,buffer,loadsize);
	  if(jpgfile) jpgfile->write(buffer,loadsize);
	  currentbuffersize -= loadsize;
	  for(size_t i = 0; i<currentbuffersize; ++i)
	    buffer[i] = buffer[i+loadsize];
	  loadsize = 0;
	}
      }
      {
	if(currentbuffersize>=sizeof(loadsize))
	{
	  // the loadsize indicates the expected size of a single buffered content. Always sizeof(size_t) bytes are used on top of the data
	  // are used, to transport the size of the following buffer. The buffer itself is transported afterwards.
	  loadsize = *((size_t*)buffer);

	  currentbuffersize -= sizeof(loadsize);
	  for(size_t i = 0; i<currentbuffersize; ++i)
	    buffer[i] = buffer[i+sizeof(loadsize)];
	  if(jpgfile)
	  {
	    jpgfile->flush();
	    jpgfile->close();
	    delete jpgfile;
	  }
	  
	  // organize a new file buffer on disk to write the splitted information jpg by jpg. The example stops after 9 written images. 
	  jpgfile = new ofstream("mview_example_jpgpart"+to_string(partnbr++)+".jpg");
	  
	  // hint: do not forget to read out and empty thereby the log from time to time if endless MJPG-streams are handled!!!! 
	  // This example is limited and short and therefore the problem not touched.
	  
	}
      }
    }
    else
    { 
      
      
            
      // it is _not_ a multipart stream, e.g. a single page 
      
      // Writes the buffer to the URL indicated by dest. 
      WriteData(dest,buffer,currentbuffersize);
      currentbuffersize = 0;
      cout << buffer;
    }
    
    
    
  }
  
 
  cout << endl;
  
  
  
  
  
  // empty the log by reading and printing it out; do not delete the pointer
  cout << "Log:" << endl << GetLog(u) << endl;
  cout << "Log:" << endl << GetLog(dest) << endl;
  
  
  // disconnect the source and the file to release the resources
  DisconnectUARU(u);
  DisconnectUARU(dest);


  
  return 0;
}
